/* *************************************************************************
   Input Interface for YAESU FTDX101MP
   Reads from the "LINEAR" interface

   The get function combines the 4 datalines to one bitmap/bitcode
   The bitcode is mapped with a table to a QRG
   This QRG can be used in the rest of the code like readed from the rig
   
   2023-08-14 
   ************************************************************************* */

#if USE_YAESU_LINEAR

/*
   this table maps the bitcode/bitmap from the input pins to a QRG
   specific for the input interface and therefore valid for all YAESU linear configurations
*/
LinearBandInformation tblLinearBandInformation[] {
  //bitcode  QRG
  {0b0001,  1810},  // 160m     
  {0b0010,  3500},  //  80m    
  //{0b0011,  5350},  //60m  similar to 40m
  {0b0011,  7000},  //  40m  similar to 60m
  {0b0100, 10100},  //  30m      
  {0b0101, 14000},  //  20m     
  {0b0110, 18068},  //  17m 
  {0b0111, 21000},  //  15m
  {0b1000, 24890},  //  12m 
  {0b1001, 28000},  //  10m
  {0b1010, 50000},  //   6m
  {0b1011, 144000}, //   2m
  {0b1011, 430000}  //  70cm
};
/*
   initialise the "linear pins" to read the "BCD"
*/
void yaesuLinearBegin() {
  for (auto &i : linearPin) pinMode(i, INPUT);
}

/*
   read the "linear pins", generate a bitmap and returns a QRG (or 0 if no new QRG or still reading)
*/
unsigned long yaesuLinearGetFrequency(void) {
  uint32_t t_QRG = 0;                            // QRG100 variable stores frequency in GMMMkkkH format - Frequency divided by 100
  static unsigned long lastfreq = 0;             // highest QRG100 in 32 bit: 4.294.967.295 --> 429 GHz (!)
  static uint32_t previousMillis = 0;
  if (millis() - previousMillis > 500) {
    previousMillis = millis();
    uint8_t bitmap = 0;
    for (uint8_t i = 0; i < 4; i++) {
      if (digitalRead(linearPin[i]) == HIGH) bitmap |= (1 << i);
    }
    t_QRG = yaesuLinear2qrg(bitmap) * 10; // this is only to be compatible to the CI-V interface
    //DEBUG_PRINT(F("D024 bitmap=")); DEBUG_PRINT(bitmap, BIN); DEBUG_PRINT(F("\tt_QRG=")); DEBUG_PRINTLN(t_QRG);
    if (t_QRG == lastfreq) {
      indicatorActivity.on();  // tbc ... pseudo activity
      return (0);              // but exit if it is the same
    }
    DEBUG_PRINT(F("D029 bitmap=")); DEBUG_PRINT(bitmap, BIN); DEBUG_PRINT(F("\tt_QRG=")); DEBUG_PRINTLN(t_QRG);
    lastfreq = t_QRG;
    return (t_QRG);
  }
  return (0); // no measurement
}

/*
   map the readed bitmap to QRG
*/
uint32_t yaesuLinear2qrg(const uint8_t bitmap) {
  constexpr uint8_t noOfLinear = sizeof(tblLinearBandInformation) / sizeof(tblLinearBandInformation[0]);
  for (uint8_t i = 0; i < noOfLinear; i++) {
    if (bitmap == tblLinearBandInformation[i].bitmap) return tblLinearBandInformation[i].qrg;
  }
  return 0;
}

#endif
